import _extends from "@babel/runtime/helpers/esm/extends";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { useState, useEffect, useCallback, useRef } from 'react';
import PropTypes from 'prop-types';
import { json } from './prop-types';
import { reducer } from './reducer';
import { RULES } from './reducer/constants';
import { pickFieldById, fieldsToObject, fieldsToObjectNativeTypes, fieldsNamesInOrderOfDefinition, changeFieldById } from './reducer/fields';
import ProxyField from './ProxyField';
import { inputSizes as fieldSizes } from '@s-ui/react-atom-input';
import AtomSpinner, { AtomSpinnerTypes } from '@s-ui/react-atom-spinner';
import { getUpdatedFormState } from './mapper/formState';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";

var FormBuilder = function FormBuilder(_ref) {
  var json = _ref.json,
      initFields = _ref.initFields,
      onChange = _ref.onChange,
      onFocus = _ref.onFocus,
      onBlur = _ref.onBlur,
      responseInterceptor = _ref.responseInterceptor,
      requestInterceptor = _ref.requestInterceptor,
      loader = _ref.loader,
      fieldSize = _ref.fieldSize,
      errors = _ref.errors,
      alerts = _ref.alerts,
      transformations = _ref.transformations,
      locale = _ref.locale,
      _ref$useNativeFieldTy = _ref.useNativeFieldType,
      useNativeFieldType = _ref$useNativeFieldTy === void 0 ? false : _ref$useNativeFieldTy,
      renderer = _ref.children;
  var _json$form = json.form,
      _json$form$fields = _json$form.fields,
      fields = _json$form$fields === void 0 ? [] : _json$form$fields,
      _json$form$rules = _json$form.rules,
      rules = _json$form$rules === void 0 ? {} : _json$form$rules,
      formID = _json$form.id;

  var _useState = useState(fields),
      stateFields = _useState[0],
      setStateFields = _useState[1];

  var _useState2 = useState(false),
      stateShowSpinner = _useState2[0],
      setStateShowSpinner = _useState2[1];

  var __PERFORMANCE_UGLY_HACK_STATE_FIELDS__ = useRef();

  __PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current = stateFields;

  var changeField = function changeField(fieldId, fieldValue) {
    var nextFields = changeFieldById(__PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current, fieldId, {
      value: fieldValue
    });
    setStateFields(nextFields);
    return nextFields;
  };

  var handlerChange = useCallback( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(id, value) {
      var reducerWithRules, timerShowSpinner, transformedValue, nextFields, nextStateFields, nextStateFieldsObject, fieldsToUpdate;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              reducerWithRules = reducer(rules, formID, responseInterceptor, requestInterceptor, locale);
              timerShowSpinner = setTimeout(function () {
                return setStateShowSpinner(true);
              }, FormBuilder.USER_MINIMAL_DELAY);
              transformedValue = transformations(id, value, pickFieldById(fields, id));
              nextFields = changeField(id, transformedValue);
              clearTimeout(timerShowSpinner);
              _context2.next = 7;
              return reducerWithRules(nextFields, {
                type: RULES,
                id: id
              });

            case 7:
              nextStateFields = _context2.sent;
              clearTimeout(timerShowSpinner);
              setStateFields(nextStateFields);
              nextStateFieldsObject = useNativeFieldType ? fieldsToObjectNativeTypes(nextStateFields) : fieldsToObject(nextStateFields);
              _context2.next = 13;
              return onChange(_extends({}, nextStateFieldsObject, {
                __FIELD_CHANGED__: id,
                __FORM_STATE__: getUpdatedFormState(json.form, nextStateFields)
              }));

            case 13:
              fieldsToUpdate = _context2.sent;
              if (typeof fieldsToUpdate === 'object' && fieldsToUpdate !== null) Object.entries(fieldsToUpdate).forEach( /*#__PURE__*/function () {
                var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref3) {
                  var key, value;
                  return _regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          key = _ref3[0], value = _ref3[1];
                          _context.next = 3;
                          return handlerChange(key, value);

                        case 3:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee);
                }));

                return function (_x3) {
                  return _ref4.apply(this, arguments);
                };
              }());
              setStateShowSpinner(false);

            case 16:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }(), []); // eslint-disable-line

  useEffect(function () {
    var reducerWithRules = reducer(rules, formID, responseInterceptor, requestInterceptor, locale);
    var timerShowSpinner = setTimeout(function () {
      return setStateShowSpinner(true);
    }, FormBuilder.USER_MINIMAL_DELAY);
    fieldsNamesInOrderOfDefinition(fields).reduce( /*#__PURE__*/function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(previousPromise, fieldId) {
        var previousFields, nextFieldsChanged, nextFieldsWithRules;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return previousPromise;

              case 2:
                previousFields = _context3.sent;

                if (!(!initFields[fieldId] || initFields[fieldId] === '')) {
                  _context3.next = 5;
                  break;
                }

                return _context3.abrupt("return", previousFields);

              case 5:
                nextFieldsChanged = changeField(fieldId, initFields[fieldId]);
                _context3.next = 8;
                return reducerWithRules(nextFieldsChanged, {
                  type: RULES,
                  id: fieldId
                });

              case 8:
                nextFieldsWithRules = _context3.sent;
                return _context3.abrupt("return", nextFieldsWithRules);

              case 10:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      return function (_x4, _x5) {
        return _ref5.apply(this, arguments);
      };
    }(), Promise.resolve(__PERFORMANCE_UGLY_HACK_STATE_FIELDS__.current)).then(function (nextFields) {
      clearTimeout(timerShowSpinner);
      setStateFields(nextFields);
      setStateShowSpinner(false);
    });
  }, []); // eslint-disable-line

  return /*#__PURE__*/_jsxs("div", {
    className: "sui-FormBuilder",
    children: [stateFields.map(function (field, index) {
      return /*#__PURE__*/_jsx(ProxyField, {
        field: field,
        tabIndex: index + 1,
        onChange: handlerChange,
        onBlur: onBlur,
        onFocus: onFocus,
        fieldSize: fieldSize,
        errors: errors,
        alerts: alerts,
        renderer: renderer
      }, field.id);
    }), stateShowSpinner && /*#__PURE__*/_jsx(AtomSpinner, {
      type: AtomSpinnerTypes.SECTION,
      loader: loader
    })]
  });
};

FormBuilder.USER_MINIMAL_DELAY = 250;
FormBuilder.displayName = 'FormBuilder';
FormBuilder.defaultProps = {
  initFields: {},
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  onFocus: function onFocus() {},
  responseInterceptor: function responseInterceptor(_ref6) {
    var response = _ref6.response;
    return response;
  },
  requestInterceptor: function requestInterceptor() {},
  errors: {},
  alerts: {},
  transformations: function transformations(_, value) {
    return value;
  },
  locale: 'es-ES',
  children: function children() {
    return {};
  }
};
export { fieldSizes as formBuilderFieldSizes };
export { pickFieldById as formBuilderPickFieldById };
export { changeFieldById as formBuilderChangeFieldById };
export { fieldsNamesInOrderOfDefinition as formBuilderFieldsNamesInOrderOfDefinition };
export { checkConstraintsFactory } from './Standard';
export default FormBuilder;